/*  File: analyser-jets.h
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 6 December 2018
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your opTion) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __analyser_jets_h__
#define __analyser_jets_h__


/*   project headers  */
#include <user.h>

/*   other includes   */
#include "local/fastjet.h"


/*   It is not a library just a module. We can use using namespace...  */
using namespace std;
using namespace duct;
using namespace duct::distpoint;


/*   Narrowing the distbook template   */
using DistArr = distbook<std::array<double, 6u>, hist1d>;


/*   
 *   This is the declaration of the analyser class.
 */
class UserJets : public basic_user<hhc, DistArr>
{
  /*   useful aliases  */
  using HistArr = Phys<DistArr>;

public:
  /*    constructors  (This uses chunks of 10M events for storing data)   */
  UserJets(const char *n, fj::JetDefinition&& jd, const duration_type& d = 2.0min)
  : jetDef{std::move(jd)} { this->param(n, d, 10000000);}
  
  UserJets(const char *n, const fj::JetDefinition& jd, const duration_type& d = 2.0min)
  : jetDef{jd} { this->param(n, d, 10000000);}
  
  /*   User defined init function. It is called once at the begining of the calculation.  */
  void initfunc();
  
  /*   The analyser routine. It analyses the events and fill the histograms.   */
  void userfunc(const shower_stage<hhc>&, const shower_stage<hhc>&, const shower_stage<hhc>&);
  
  /*   For fastjet analysis - select algorithm and parameters.    */
  fj::JetDefinition jetDef;
  
  /*   Predefined vectors of Pseudojets     */
  mutable std::vector<fj::PseudoJet> theparticles;
  mutable std::vector<fj::PseudoJet> incJets;
};

#endif


