/*  File: analyser-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 23 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   local headers  */
#include "analyzer-phase.h"



void UserPhase::initfunc()
{
  /*   Unitary test    */
  HistArr6::phys(1, "Unitary test", spacing<hist1d>::linear(25, 0.0, 10.0));
  HistArr27::phys(1, "Unitary test (partial contributions)", spacing<hist1d>::linear(25, 0.0, 10.0));
}



void UserPhase::userfunc(const shower_stage<hhc>& hard, const shower_stage<hhc>&, const shower_stage<hhc>& showered)
{
  //=================== Unitary condition   =============================
  std::array<double, 6> weights;
  std::array<double, 27> partial;

  weights.fill(0.0);
  partial.fill(0.0);

  /*   the zeroth element is the Born cross section    */
  weights[0] = partial[0] = weight(hard);
 
  
  /*   showered cross section    */
  auto func = [&weights, &partial](const partial_weight_t& w)
  {
    /*    we ignore threshold and delta corrections    */
    if(w.deltaT != 0u) return;

    
    /*  pure LC+ contribution    */
    if(w.phase == 0u)
    {
      weights[1] += w.wstd;
    }
    
    /*   first order   */
    else if(w.phase == 1u)
    {
      weights[2] += w.wstd;
      
      if(w.phaseL == 1u) partial[1] += w.wstd;
      if(w.phaseR == 1u) partial[2] += w.wstd;
      if(w.phaseU == 1u) partial[3] += w.wstd;
    }
    
    /*   two insertions of delta operators   */
    else if(w.phase == 2u)
    {
      weights[3] -= w.wstd;

      if(w.phaseR == 2u)                   partial[4] -= w.wstd;
      if(w.phaseL == 2u)                   partial[4] -= w.wstd;
      if(w.phaseR == 1u && w.phaseL == 1u) partial[5] -= w.wstd;
      
      if(w.phaseU == 2u)                   partial[6] -= w.wstd;
      if(w.phaseU == 1u && w.phaseL == 1u) partial[7] -= w.wstd;
      if(w.phaseU == 1u && w.phaseR == 1u) partial[7] -= w.wstd;
    }
   
    /*   three insertions of delta operators   */
    else if(w.phase == 3u)
    {
      weights[4] -= w.wstd;
      
      if(w.phaseL == 3u)                   partial[8] -= w.wstd;
      if(w.phaseR == 3u)                   partial[9] -= w.wstd;
      if(w.phaseL == 2u && w.phaseR == 1u) partial[10] -= w.wstd;
      if(w.phaseR == 2u && w.phaseL == 1u) partial[11] -= w.wstd;
     
      if(w.phaseU == 3u)                   partial[12] -= w.wstd;
      if(w.phaseU == 2u && w.phaseL == 1u) partial[13] -= w.wstd;
      if(w.phaseU == 2u && w.phaseR == 1u) partial[14] -= w.wstd;

      if(w.phaseU == 1u && w.phaseL == 2u)                   partial[15] -= w.wstd;
      if(w.phaseU == 1u && w.phaseR == 2u)                   partial[16] -= w.wstd;
      if(w.phaseU == 1u && w.phaseL == 1u && w.phaseR == 1u) partial[17] -= w.wstd;
      
    }
    
    /*   four insertions of delta operators   */
    else if(w.phase == 4u)
    {
      weights[5] += w.wstd;
      
      if(w.phaseL == 4u)                   partial[18] += w.wstd;
      if(w.phaseR == 4u)                   partial[18] += w.wstd;
      if(w.phaseL == 3u && w.phaseR == 1u) partial[19] += w.wstd;
      if(w.phaseL == 1u && w.phaseR == 3u) partial[19] += w.wstd;
      if(w.phaseL == 2u && w.phaseR == 2u) partial[20] += w.wstd;
      
      if(w.phaseU == 4u)                   partial[21] += w.wstd;
      
      if(w.phaseU == 3u && w.phaseL == 1u) partial[22] += w.wstd;
      if(w.phaseU == 3u && w.phaseR == 1u) partial[22] += w.wstd;
      
      if(w.phaseU == 2u && w.phaseL == 2u)                   partial[23] += w.wstd;
      if(w.phaseU == 2u && w.phaseR == 2u)                   partial[23] += w.wstd;
      if(w.phaseU == 2u && w.phaseL == 1u && w.phaseR == 1u) partial[24] += w.wstd;
      
      if(w.phaseU == 1u && w.phaseL == 3u)                   partial[25] += w.wstd;
      if(w.phaseU == 1u && w.phaseR == 3u)                   partial[25] += w.wstd;
      if(w.phaseU == 1u && w.phaseL == 2u && w.phaseR == 1u) partial[26] += w.wstd;
      if(w.phaseU == 1u && w.phaseL == 1u && w.phaseR == 2u) partial[26] += w.wstd;
     
    }
  };
  
  
  /*    accumulating the weights   */
  analyse_color_weights(showered, std::move(func));
  
  
  /*    fill the histogram    */
  double tmH = static_cast<const shower_history_stage<hhc>&>(hard).current_time;
  
  HistArr6::physfill(1, weights, dirac{}, tmH);
  HistArr27::physfill(1, partial, dirac{}, tmH);
}

