/*  File: mod-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   Deductor headers  */
#include <deductor.h>
#include <proc-hhcjets.h>


/*   Analyser headers  */
#include "analyzer-phase.h"


auto my_jet_module(const deductor_input&)
{
  /*  Using a previously creating the kernel   */
  deductor<hhc> kernel{qcd::ct14n};
  
  /*    Hard process     */
  auto hard = [=](const model_ref& mdl) -> hhcjets
  {
    int proton = 2212;
    double Ecm = 13.0_TeV;
    double scalefac_pert = 0.707;
    double scalefac_shw = 1.5;
    
    auto pTlist = {
      20.0, 50.0, 100.0, 200.0, 400.0, 600.0, 800.0, 1200.0, 1600.0,
      2000.0, 2500.0, 3000.0, 3500.0, 4000.0, 4500.0, 5000.0
    };
    
    return {mdl, Ecm, pTlist, scalefac_pert, scalefac_shw, proton, proton};
  };
  
  
  /*   Shower options */
  shower_engine_opts opts;
  
  opts.kT_cutoff = 5.0_GeV;
  opts.threshold.enabled = false;
  opts.threshold.order_of_soft_corrections = 0u;
  opts.is_insane = true;
  
  /*   controling color approximation     */
  // This applies for t_H < t < t_H + 5.
  opts.color[5.0] = {
    {UINT_MAX, 4u, {4u, 0u, 4u}},
  };
  
  /*    Starting the calculation    */
  cout<<opts.sanitize()<<"\n";
  kernel(ordering::lambda{}, {"JetCalculations", opts}, hard, new UserPhase{"UnitarityTest"});
}


/*   Always export your module functions!   */
export_my_module jets_module = {
  {"UnitarityTestLHC13TeV", my_jet_module}
};

